class Gps:
    def __init__(self, gps_message):
        self.message = gps_message
        self.timestamp = self.message._timestamp
        self.lat = self.message.lat
        self.lon = self.message.lon
        self.alt = self.message.alt
        self.vel = self.message.vel

    def retake_lat(self):
        self.lat = self.message.lat

    def get_lat(self):
        self.retake_lat()
        return self.lat

    def retake_vel(self):
        self.vel = self.message.vel

    def get_vel(self):
        self.retake_vel()
        return self.vel

    def retake_lon(self):
        self.lon = self.message.lon

    def get_lon(self):
        self.retake_lon()
        return self.lon

    def retake_alt(self):
        self.alt = self.message.alt

    def get_alt(self):
        self.retake_alt()
        return self.alt

    def retake_timestamp(self):
        self.timestamp = self.message.timestamp

    def get_timestamp(self):
        self.retake_timestamp()
        return self.timestamp



class Error(Exception):
    pass


class ConnectionIsNoneError(Error):
    def __init__(self, connection_string, message="is not connection_string"):
        self.connection_string = connection_string
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'\n\t{self.connection_string} {self.message}'

    def is_connection_none(self):
        if self.connection_string is None:
            raise ConnectionIsNoneError(self.connection_string)



import time

from planekit.mavlink.Connection import Connection as MavLinkConnection
from planekit.mavlink.SendMessage import SendMessage
from planekit.data.Imu import Imu
from planekit.mavlink.ReceiveData import ReceiveData
from planekit.command.SetArm import SetArm
from planekit.command.FightMode import FlightMode
from planekit.command.TakeOffLandCMD import TakeOffLandCMD
from planekit.command.DoReposition import DoReposition
from planekit.Calculator import Calculator


class Connection:
    def __init__(self, connection_string):
        self.mavlink = MavLinkConnection()
        # master
        self.connection_object = self.mavlink.connection(connection_string)
        self.connection_string = connection_string

    def wait_heartbeat(self):
        m = SendMessage(self.connection_object)
        m.wait_heartbeat()

    # Test remove prod.
    def get_connection(self):
        self.mavlink.get_connection()

    def imu_message(self):
        return ReceiveData(self.connection_object).select_imu_message()

    def gps_message(self):
        return ReceiveData(self.connection_object).select_gps_message()

    def heartbeat_message(self):
        return ReceiveData(self.connection_object).select_heartbeat_message()

    def ahrs_message(self):
        return ReceiveData(self.connection_object).select_ahrs_message()

    def lat(self):
        return ReceiveData(self.connection_object).select_gps_message().lat/10000000

    def lon(self):
        return ReceiveData(self.connection_object).select_gps_message().lon/10000000

    def alt(self):
        return ReceiveData(self.connection_object).alt()

    def ground_speed(self):
        return ReceiveData(self.connection_object).ground_speed_message()/100

    def arm_status(self):
        return ReceiveData(self.connection_object).arm_status()

    def mode(self):
        return FlightMode(self.connection_object).get_flight_mode()

    def arm(self):
        set_arm = SetArm(self.connection_object)
        set_arm.arm()

    def disarm(self):
        set_arm = SetArm(self.connection_object)
        set_arm.disarm()

    def set_flight_mode(self, mode):
        set_flight_modes = FlightMode(self.connection_object)
        set_flight_modes.set_flight_mode(mode)

    def takeoff_and_arm(self, alt):
        takeoff = TakeOffLandCMD(self.connection_object)
        takeoff.takeoff_string(alt)
        self.arm()
        self.set_flight_mode("AUTO")
        while self.mode() != "AUTO":
            #print(self.mode())
            self.set_flight_mode("AUTO")
            time.sleep(10)

    def land_and_disarm(self):
        while self.arm_status() is True:
            land = TakeOffLandCMD(self.connection_object)
            land.lend_string(12)
            # print(self.mode())
            print("landing")
            self.set_flight_mode("AUTO")
            time.sleep(10)

    def go_to(self, lat, lon, alt, sleep=False):
        reposition = DoReposition(self.connection_object)
        reposition.go_waypoint(lat, lon, alt)
        while sleep:
            a = self.gps_message()
            if Calculator.haversine(-35.3630510, 149.1654968, a.lat / 10000000, a.lon / 10000000) < 0.200:
                break




from distutils.core import setup
import setuptools


VERSION = "0.1.905"  # BURAYI AKLINIZDA TUTUN (Değiştirebilirsiniz)
long_description = ""
with open("README.md", "r", encoding="utf-8") as file:
    long_description = file.read()
setup(
    name='planekit',
    packages=setuptools.find_packages(),
    install_requires=['pymavlink==2.4.29', 'pyserial==3.5'],
    options={"bdist_wheel": {"universal": True}},
    version='0.1.905',
    license='MIT',
    description='Developer Tools for Fixed-wing',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Musa Şen',
    author_email='m42@gmail.com',
    url='https://github.com/msasen/planekit',

    download_url=f'https://github.com/msasen/planekit{VERSION}.tar.gz',
    keywords=['Planekit', 'plane', 'autonomous', 'fixed-wing'],

)



class Imu:
    def __init__(self, imu_message):
        self.message = imu_message
        # print(self.message)
        self.xacc = self.message.xacc
        self.yacc = self.message.yacc
        self.zacc = self.message.zacc
        self.xgyro = self.message.xgyro
        self.ygyro = self.message.ygyro
        self.zgyro = self.message.zgyro
        self.xmag = self.message.xmag
        self.ymag = self.message.ymag
        self.zmag = self.message.zmag

    def retake_xacc(self):
        self.xacc = self.message.xacc

    def get_xacc(self):
        self.retake_xacc()
        return self.xacc

    def retake_yacc(self):
        self.yacc = self.message.yacc

    def get_yacc(self):
        self.retake_yacc()
        return self.yacc

    def retake_zacc(self):
        self.zacc = self.message.zacc

    def get_zacc(self):
        self.retake_zacc()
        return self.zacc

    def retake_xgyro(self):
        self.xgyro = self.message.xgyro

    def get_xgyro(self):
        self.retake_xgyro()
        return self.xgyro

    def retake_ygyro(self):
        self.ygyro = self.message.ygyro

    def get_ygyro(self):
        self.retake_ygyro()
        return self.ygyro

    def retake_zgyro(self):
        self.zgyro = self.message.zgyro

    def get_zgyro(self):
        self.retake_zgyro()
        return self.zgyro

    def retake_xmag(self):
        self.xmag = self.message.xmag

    def get_xmag(self):
        self.retake_xmag()
        return self.xmag

    def retake_ymag(self):
        self.ymag = self.message.ymag

    def get_ymag(self):
        self.retake_ymag()
        return self.ymag

    def retake_zmag(self):
        self.zmag = self.message.zmag

    def get_zmag(self):
        self.retake_zmag()
        return self.zmag
